
import ai.catboost.CatBoostModel;
import ai.catboost.CatBoostPredictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.Map;
 
public class CatBoostClassifier {
 
    public CatBoostModel model;
 
    private static Logger LOG= LoggerFactory.getLogger(CatBoostClassifier.class);
 
    public CatBoostClassifier(String model_path) {
        try{

            InputStream model_file = new FileInputStream(new File(model_path));
            this.model = CatBoostModel.loadModel(model_file);
 
        }catch (Exception err){
            LOG.error("catboost-init-error", err);
            System.out.println(err.toString());
        };
 
    }
 
    public Double predict(Map<String,Double> num_features,Map<String,String> cat_features){
        try {
            String[] feature_names = this.model.getFeatureNames();
            assert (num_features.size() + cat_features.size()) == feature_names.length;
 
            float[] num_arr = new float[num_features.size()];
            String[] cat_arr = new String[cat_features.size()];
 
            int i = 0;
            int j = 0;
            for (String name : feature_names) {
                if (num_features.keySet().contains(name)) {
                    num_arr[i] = num_features.get(name).floatValue();
                    i += 1;
                } else {
                    assert cat_features.keySet().contains(name);
                    cat_arr[j] = cat_features.get(name);
                    j += 1;
                }
            }
            CatBoostPredictions prediction = this.model.predict(
                    num_arr,
                    cat_arr);
            Double prob = 1.0 - 1.0 / (1.0 + Math.exp(prediction.get(0, 0)));
            return prob;
        }catch (Exception err){
            LOG.error("catboost-predict-error", err);
            return null;
        }
    }
}