import org.jpmml.evaluator.*;
import org.jpmml.evaluator.visitors.DefaultModelEvaluatorBattery;
import org.jpmml.model.visitors.VisitorBattery;
import org.xml.sax.SAXException;
import org.dmg.pmml.FieldName;

import javax.xml.bind.JAXBException;
import java.io.File;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class Server {

    public static void main(String[] args) throws JAXBException, SAXException, IOException {
        // Building a model evaluator from a PMML file
        String modelPath = "/Users/alvin/Downloads/first_test.pmml";
        //String modelPath = "lgb_model.pmml";
        //System.out.println(modelPath);
        Evaluator evaluator = new LoadingModelEvaluatorBuilder()
                .setLocatable(false)
                .load(new File(modelPath)).build();

        // Performing the self-check
        evaluator.verify();

        // Printing input (x1, x2, .., xn) fields
        List<? extends InputField> inputFields = evaluator.getInputFields();
        System.out.println("Input fields: ");
        for (InputField inputField : inputFields) {
            System.out.println(inputField);
        }

        // Printing primary result (y) field(s)
        List<? extends TargetField> targetFields = evaluator.getTargetFields();
        System.out.println("Target field(s): " + targetFields);

        // Printing secondary result (eg. probability(y), decision(y)) fields
        List<? extends OutputField> outputFields = evaluator.getOutputFields();
        System.out.println("Output fields: ");
        for (OutputField outputField : outputFields) {
            System.out.println(outputField);
        }

        List<String> strings = FileUtil.readToLines("/Users/alvin/Downloads/11111111111111111111111111111111111111.csv");
        for (String string : strings) {
            // Predicting
            Float[] doubles = new Float[23];
            String[] split = string.split(",");
            for (int i = 0; i < split.length; i++) {
                if (split[i].length() == 0) {
                    doubles[i] = new Float(0);
                } else {
                    doubles[i] = new Float(split[i]);
                }
            }
            Float[] arr = new Float[]{5.1f, 3.5f, 1.4f, 0.2f};

            Map<FieldName, FieldValue> arguments = new LinkedHashMap<FieldName, FieldValue>();

            // Mapping the record field-by-field from data source schema to PMML schema
            for (int i = 0; i < inputFields.size(); i++) {
                InputField inputField = inputFields.get(i);
                arguments.put(inputField.getName(), inputField.prepare(doubles[i]));
            }

            // Evaluating the model with known-good arguments
            Map<FieldName, ?> results = evaluator.evaluate(arguments);
            //System.out.println(results);

            // Decoupling results from the JPMML-Evaluator runtime environment
            Map<String, ?> resultRecord = EvaluatorUtil.decodeAll(results);
            System.out.println(resultRecord);
            break;
        }


    }
}
